package driver.communication;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.charset.Charset;

import gnu.io.CommPort;
import gnu.io.CommPortIdentifier;
import gnu.io.SerialPort;

public class SerialPortCommunicationManager implements CommunicationManager {

	private int timeout = 2000;
	private String portIdetifier = "COM1";
	private String portName = "COM1";
	private int baudrate = 9600;
	private int dataBits = SerialPort.DATABITS_8;
	private int stopBits = SerialPort.STOPBITS_1;
	private int parity = SerialPort.PARITY_NONE;
	private Charset charset = Charset.forName("US-ASCII");
	private String eocString = "<EOC>";

	private SerialPort serialport = null;
	private InputStream is = null;
	private OutputStream os = null;

	private boolean isConnected = false;

	public SerialPortCommunicationManager() {}

	public SerialPortCommunicationManager(String portIdetifier, String portName, int baudrate, int dataBits, int stopBits, int parity, Charset charset, String eocString) {
		this.portIdetifier = (portIdetifier != null && !portIdetifier.equals("")) ? portIdetifier : this.portIdetifier;
		this.portName = (portName != null && !portName.equals("")) ? portName : this.portName;;
		this.baudrate = baudrate;
		this.dataBits = dataBits;
		this.stopBits = stopBits;
		this.parity = parity;
		this.charset = (charset != null) ? charset : this.charset;
		this.eocString = (eocString != null && !eocString.equals("")) ? eocString : this.eocString;
	}

	@Override
	public boolean connect() throws CommunicationException {

		try {
			if(this.isConnected == false) {
				CommPortIdentifier cpi = CommPortIdentifier.getPortIdentifier(this.portIdetifier);
				CommPort cp = cpi.open(this.portName,this.timeout);
				if(cp instanceof SerialPort) {
					this.serialport = (SerialPort) cp;
				} else {
					throw new CommunicationException("[SerialPortCommunicationManager]-->[connect()]: CommPort not instance of SerialPort");
				}
				this.serialport.setSerialPortParams(this.baudrate,this.dataBits,this.stopBits,this.parity);
			}
		} catch(Exception e) {
			throw new CommunicationException(e.getMessage());
		}

		this.isConnected = true;

		return true;
	}

	@Override
	public String read() throws CommunicationException {

		String str = "";

		try {
			if(this.isConnected == true) {
				if(this.is == null) {
					this.is = new BufferedInputStream(this.serialport.getInputStream());
				}
				do {
					str = str.concat(Character.toString((char) this.is.read()));
				} while(str.indexOf(this.eocString) == -1);
				str = str.substring(0, str.indexOf(this.eocString));
			} else {
				throw new CommunicationException("[SerialPortCommunicationManager]-->[read()]: CommunicationManager not connected");
			}
		} catch(Exception e) {
			throw new CommunicationException(e.getMessage());
		}

		return str;
	}

	public String read(Charset charset, String eocString) throws CommunicationException {

		String str = "";

		try {
			if(this.isConnected == true) {
				if(this.is == null) {
					this.is = new BufferedInputStream(this.serialport.getInputStream());
				}
				do {
					str = str.concat(Character.toString((char) this.is.read()));
				} while(str.indexOf(eocString) == -1);
				if(!eocString.equals("")) {
					str = str.substring(0, str.indexOf(this.eocString));
				}
			} else {
				throw new CommunicationException("[SerialPortCommunicationManager]-->[read(Charset charset, String eocString)]: CommunicationManager not connected");
			}
		} catch(Exception e) {
			throw new CommunicationException(e.getMessage());
		}

		return str;
	}

	@Override
	public boolean write(String str) throws CommunicationException {

		try {
			if(this.isConnected == true) {
				if(this.os == null) {
					this.os = new BufferedOutputStream(this.serialport.getOutputStream());
				}
				this.os.write((str.concat(this.eocString)).getBytes(this.charset));
				this.os.flush();
			} else {
				throw new CommunicationException("[SerialPortCommunicationManager]-->[write(String str)]: CommunicationManager not connected");
			}
		} catch(Exception e) {
			throw new CommunicationException(e.getMessage());
		}

		return true;
	}

	public boolean write(String str, Charset charset, String eocString) throws CommunicationException {

		try {
			if(this.isConnected == true) {
				if(this.os == null) {
					this.os = new BufferedOutputStream(this.serialport.getOutputStream());
				}
				this.os.write((str.concat(eocString)).getBytes(charset));
				this.os.flush();
			} else {
				throw new CommunicationException("[SerialPortCommunicationManager]-->[write(String str, Charset charset, String eocString)]: CommunicationManager not connected");
			}
		} catch(Exception e) {
			throw new CommunicationException(e.getMessage());
		}

		return true;
	}

	@Override
	public boolean disconnect() throws CommunicationException {

		try {
			this.is.close();
			this.os.close();
			this.serialport.close();
			this.is = null;
			this.os = null;
			this.serialport = null;
			this.isConnected = false;
		} catch(Exception e) {
			throw new CommunicationException(e.getMessage());
		}

		return true;
	}

}
