package driver.communication;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.nio.charset.Charset;

public class SocketCommunicationManager implements CommunicationManager {

	private String targetHost = "localhost";
	private int targetPort = 11111;
	private Charset charset = Charset.forName("US-ASCII");
	private String eocString = "<EOC>";

	private Socket socket = null;
	private InputStream is = null;
	private OutputStream os = null;

	private boolean isConnected = false;

	public SocketCommunicationManager() {}

	public SocketCommunicationManager(String targetHost, int targetPort, Charset charset, String eocString) {
		this.targetHost = (targetHost != null && !targetHost.equals("")) ? targetHost : this.targetHost;
		this.targetPort = targetPort;
		this.charset = (charset != null) ? charset : this.charset;
		this.eocString = (eocString != null && !eocString.equals("")) ? eocString : this.eocString;
	}

	@Override
	public boolean connect() throws CommunicationException {

		try {
			if(this.isConnected == false) {
				this.socket = new Socket(this.targetHost, this.targetPort);
			}
		} catch(Exception e) {
			throw new CommunicationException(e.getMessage());
		}

		this.isConnected = true;

		return true;
	}

	@Override
	public String read() throws CommunicationException {

		String str = "";

		try {
			if(this.isConnected == true) {
				if(this.is == null) {
					this.is = new BufferedInputStream(this.socket.getInputStream());
				}
				do {
					str = str.concat(Character.toString((char) this.is.read()));
				} while(str.indexOf(this.eocString) == -1);
				str = str.substring(0, str.indexOf(this.eocString));
			} else {
				throw new CommunicationException("[SocketCommunicationManager]-->[read()]: CommunicationManager not connected");
			}
		} catch(Exception e) {
			throw new CommunicationException(e.getMessage());
		}

		return str;
	}

	@Override
	public boolean write(String str) throws CommunicationException {

		try {
			if(this.isConnected == true) {
				if(this.os == null) {
					this.os = new BufferedOutputStream(this.socket.getOutputStream());
				}
				this.os.write((str.concat(this.eocString)).getBytes(this.charset));
				this.os.flush();
			} else {
				throw new CommunicationException("[SocketCommunicationManager]-->[write(String str)]: CommunicationManager not connected");
			}
		} catch(Exception e) {
			throw new CommunicationException(e.getMessage());
		}

		return true;
	}

	@Override
	public boolean disconnect() throws CommunicationException {

		try {
			this.is.close();
			this.os.close();
			this.socket.close();
			this.is = null;
			this.os = null;
			this.socket = null;
			this.isConnected = false;
		} catch(Exception e) {
			throw new CommunicationException(e.getMessage());
		}

		return true;
	}

}
