package driver.core;

import java.nio.charset.Charset;
import java.util.Properties;

import driver.communication.CommunicationException;
import driver.communication.CommunicationManager;
import driver.communication.SerialPortCommunicationManager;
import driver.communication.SocketCommunicationManager;

public class ETMDCycletteCoreDriver implements CoreDriver {

	private Properties props = null;

	private CommunicationManager serialPortCM = null;
	private CommunicationManager socketCM = null;
	private EDProtocol protocol = new EDProtocol("0","0");

	private static final String sSTART = "S";
	private static final String sEND = "F";
	private static final String wPOWER = "W";
	private static final String rRPM = "D";

	private static final int powerStep = 5;

	public ETMDCycletteCoreDriver() {}

	public ETMDCycletteCoreDriver(Properties props) {
		this.props = props;
	}

	@Override
	public boolean start() throws CoreException {

		try {
			if(this.connectSerialPort() & this.connectSocket()) {
				this.serialPortCM.write(ETMDCycletteCoreDriver.sSTART);
			} else {
				return false;
			}
		} catch(Exception e) {
			throw new CoreException(e.getMessage());
		}

		return true;
	}

	@Override
	public void driveAndSync() throws CoreException {

		try {
			String read = "";
			do {
				read = this.socketCM.read();

				if(Integer.toString(EDProtocol.EOR).equals(read)) {
					break;
				}

				this.protocol = new EDProtocol(read);

				float resistance = Float.parseFloat(this.protocol.getResistance());
				int power = this.getPowerFromResistance(resistance);

				this.serialPortCM.write(ETMDCycletteCoreDriver.wPOWER.concat(String.format("%03d", power)));
				this.serialPortCM.write(ETMDCycletteCoreDriver.rRPM);

				int rpm = Integer.parseInt(this.serialPortCM.read().substring(1));
				rpm = rpm > 90 ? 90 : rpm;
				float velocity = this.getVelocityFromRPM(rpm);

				this.protocol.setVelocity(Float.toString(velocity));
				this.socketCM.write(this.protocol.getProtocolString());

				Thread.currentThread().sleep(Integer.parseInt(this.props.getProperty("etmdcyclettecoredriver.driver.core.refreshdelay")));
			} while(true);
		} catch(Exception e) {
			throw new CoreException(e.getMessage());
		}

	}

	@Override
	public boolean end() throws CoreException {

		try {
			this.serialPortCM.write(ETMDCycletteCoreDriver.sEND);
			if(this.socketCM.disconnect() & this.serialPortCM.disconnect()) {
				//TODO: end
			} else {
				return false;
			}
		} catch(Exception e) {
			throw new CoreException(e.getMessage());
		}

		return true;
	}

	private boolean connectSerialPort() throws CommunicationException {

		this.serialPortCM = new SerialPortCommunicationManager(
					this.props.getProperty("etmdcyclettecoredriver.driver.communication.serialport.portidetifier"),
					this.props.getProperty("etmdcyclettecoredriver.driver.communication.serialport.portname"),
					Integer.parseInt(this.props.getProperty("etmdcyclettecoredriver.driver.communication.serialport.baudrate")),
					Integer.parseInt(this.props.getProperty("etmdcyclettecoredriver.driver.communication.serialport.dataBits")),
					Integer.parseInt(this.props.getProperty("etmdcyclettecoredriver.driver.communication.serialport.stopBits")),
					Integer.parseInt(this.props.getProperty("etmdcyclettecoredriver.driver.communication.serialport.parity")),
					Charset.forName(this.props.getProperty("etmdcyclettecoredriver.driver.communication.serialport.charset")),
					this.props.getProperty("etmdcyclettecoredriver.driver.communication.serialport.eocstring")
				);

			return this.serialPortCM.connect();

	}

	private boolean connectSocket() throws CommunicationException {

		this.socketCM = new SocketCommunicationManager(
					this.props.getProperty("etmdcyclettecoredriver.driver.communication.socket.targethost"),
					Integer.parseInt(this.props.getProperty("etmdcyclettecoredriver.driver.communication.socket.targetport")),
					Charset.forName(this.props.getProperty("etmdcyclettecoredriver.driver.communication.socket.charset")),
					this.props.getProperty("etmdcyclettecoredriver.driver.communication.socket.eocstring")
				);

			return this.socketCM.connect();

	}

	private int getPowerFromResistance(float resistance) {
		int power = Math.round(Integer.parseInt(this.props.getProperty("etmdcyclettecoredriver.driver.core.maxpower")) * resistance / Integer.parseInt(this.props.getProperty("etmdcyclettecoredriver.driver.core.maxresistance")));
		power = Math.round(power / ETMDCycletteCoreDriver.powerStep) * ETMDCycletteCoreDriver.powerStep;
		return power;
	}

	private float getVelocityFromRPM(int rpm) {
		float velocity = rpm / Integer.parseInt(this.props.getProperty("etmdcyclettecoredriver.driver.core.rpmmsratio"));
		return velocity;
	}
}
