package driver.main;

import java.io.File;
import java.io.FileInputStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;

import driver.constants.DriverConst;
import driver.core.CoreDriver;

public class Main {

	public static void main(String[] args) {

		String cfgFileStr = (args[0] != null && !args[0].equals("")) ? args[0] : DriverConst.DRIVER_MAIN_CONFIGURATION_FILE;		

		File cfgFile = new File(cfgFileStr);

		if(cfgFile.exists() && !cfgFile.isDirectory()) {
			DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
			Date date = new Date();
			System.out.println("[Main]-->[main()]: Starting driver at " + dateFormat.format(date));

			CoreDriver driver = null;

			try {
				Properties props = new Properties();
				props.load(new FileInputStream(cfgFile));
				driver = (CoreDriver) Class.forName(props.getProperty(DriverConst.PROP_DRIVER_CORE_CLASSNAME)).getConstructor(Properties.class).newInstance(props);
				if(driver.start()) {
					driver.driveAndSync();
				} else {
					System.err.println("[Main]-->[main()]: ERROR: Driver not started");
					System.out.println("[Main]-->[main()]: ERROR: Driver not started");
				}
			} catch(Exception e) {
				System.err.println("[Main]-->[main()]: ERROR: Unhandled Exception: " + e.getMessage());
				System.err.println("[Main]-->[main()]: ERROR: View log for details");
				System.out.println("[Main]-->[main()]: ERROR: Unhandled Exception: " + e.getMessage());
				for(StackTraceElement ste : e.getStackTrace()) {
					System.out.println(ste.toString());
				}
			} finally {
				if(driver != null) {
					try {
						if(!driver.end()) {
							System.err.println("[Main]-->[main()]: ERROR: Driver not ended correctly");
							System.out.println("[Main]-->[main()]: ERROR: Driver not ended correctly");
						}
					} catch(Exception e) {
						System.err.println("[Main]-->[main()]: ERROR: Unhandled Exception: " + e.getMessage());
						System.err.println("[Main]-->[main()]: ERROR: View log for details");
						System.out.println("[Main]-->[main()]: ERROR: Unhandled Exception: " + e.getMessage());
						for(StackTraceElement ste : e.getStackTrace()) {
							System.out.println(ste.toString());
						}
					}
				}
			}

			date = new Date();
			System.out.println("[Main]-->[main()]: Driver ended at " + dateFormat.format(date));

			System.exit(0);
		} else {
			System.err.println("[Main]-->[main()]: ERROR: " + cfgFileStr + " is not a valid configuration file");
			System.out.println("[Main]-->[main()]: ERROR: " + cfgFileStr + " is not a valid configuration file");
			System.exit(1);
		}

	}

}
